#!/usr/bin/python

# This file is part of the Zorin Web Browser Manager program.
#
# Copyright 2016 Zorin OS Technologies Ltd.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms and conditions of the GNU General Public License,
# version 3, as published by the Free Software Foundation.
#
# This program is distributed in the hope it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.

import gi
import listbox, apt, sys
from aptdaemon.client import AptClient
from aptdaemon.gtk3widgets import AptErrorDialog, AptProgressDialog
import aptdaemon.errors
from aptdaemon.enums import *
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gio, GLib
import apt_pkg
import gettext

apt_client = AptClient()

t = gettext.translation('aptdaemon', 'usr/share/locale', fallback=True)
_ = t.ugettext

class InstallationButton(Gtk.Button):
    def __init__ (self, package_name, main_window):
        Gtk.Button.__init__(self)

        self.package_name = package_name
        self.main_window = main_window

        self.openCache()

        if self.cache.has_key(self.package_name) and self.cache[self.package_name].is_installed:
            self.set_label(_("Remove"))
            self.signal_id = self.connect("clicked", self.on_remove)
        else:
            self.set_label(_("Install"))
            self.signal_id = self.connect("clicked", self.on_install)

        self.set_size_request(80, -1)

    def openCache(self):
        try:
            self.cache = apt.Cache()
        except SystemError as strerr:
            if not '/etc/apt/sources.list' in str(strerr):
                raise
            dialog = Gtk.MessageDialog(transient_for=self.main_window,
                                       message_type=Gtk.MessageType.ERROR,
                                       buttons=Gtk.ButtonsType.OK,
                                       text="Invalid /etc/apt/sources.list file")
            dialog.format_secondary_text(strerr)
            dialog.run()
            dialog.destroy()
            sys.exit(0)
        if self.cache._depcache.broken_count > 0:
            err_header = "Software index is broken"
            err_body = "This is a major failure of your software "
            "management system. Please check for broken packages "
            "with synaptic, check the file permissions and "
            "correctness of the file '/etc/apt/sources.list' and "
            "reload the software information with: "
            "'sudo apt-get update' and 'sudo apt-get install -f'."
            dialog = Gtk.MessageDialog(transient_for=self.main_window,
                                       message_type=Gtk.MessageType.ERROR,
                                       buttons=Gtk.ButtonsType.OK,
                                       text=err_header)
            dialog.format_secondary_text(err_body)
            dialog.run()
            dialog.destroy()
            sys.exit(0)
        return True

    def on_error(self, error):
        if isinstance(error, aptdaemon.errors.NotAuthorizedError):
            # Silently ignore auth failures
            return
        elif not isinstance(error, aptdaemon.errors.TransactionFailed):
            # Catch internal errors of the client
            error = aptdaemon.errors.TransactionFailed(ERROR_UNKNOWN,str(error))
        dia = AptErrorDialog(error, parent=self.main_window)
        dia.run()
        dia.hide()

    def on_finished_update(self, trans, exit):
        if exit == "exit-success":
            GLib.timeout_add(200, self.do_install)
        return True

    def do_update(self):
        trans_update = apt_client.update_cache()
        trans_update.connect("finished", self.on_finished_update)

        dia = AptProgressDialog(trans_update, parent=self.main_window)
        dia.run(close_on_finished=True,
                show_error=True,
                reply_handler=lambda: True,
                error_handler=self.on_error)
        return False

    def on_install(self, widget):
        if not self.cache.has_key(self.package_name):
            self.do_update()
        else:
            self.do_install()

    def do_install(self):
        trans_inst = apt_client.install_packages([self.package_name])
        trans_inst.connect("finished", self.on_finished_install)
        dia = AptProgressDialog(trans_inst, parent=self.main_window)
        dia.run(close_on_finished=True,
                show_error=True,
                reply_handler=lambda: True,
                error_handler=self.on_error)
        return False

    def on_finished_install(self, trans, exit):
        if exit == "exit-success":
            self.set_label("Remove")
            self.disconnect(self.signal_id)
            self.signal_id = self.connect("clicked", self.on_remove)
            GLib.timeout_add(200, self.finished_dialog, "Installed")
        return True

    def on_remove(self, widget):
        trans_remo = apt_client.remove_packages([self.package_name])
        trans_remo.connect("finished", self.on_finished_remove)
        dia = AptProgressDialog(trans_remo, parent=self.main_window)
        dia.run(close_on_finished=True,
                show_error=True,
                reply_handler=lambda: True,
                error_handler=self.on_error)
        return

    def on_finished_remove(self, trans, exit):
        if exit == "exit-success":
            self.set_label("Install")
            self.disconnect(self.signal_id)
            self.signal_id = self.connect("clicked", self.on_install)
            GLib.timeout_add(200, self.finished_dialog, "Removed")
        return True

    def finished_dialog(self, message):
        dialog = Gtk.MessageDialog(transient_for=self.main_window,
                                   message_type=Gtk.MessageType.INFO,
                                   buttons=Gtk.ButtonsType.OK,
                                   text=message)
        dialog.run()
        dialog.destroy()
        return False

class BrowserRow(listbox.ListBoxRow):
    def __init__ (self, name, icon_name, package_name, main_window):

        icon = Gio.ThemedIcon(name=icon_name)
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.DIALOG)

        button = InstallationButton(package_name, main_window)
        button.props.vexpand = False
        button.props.valign = Gtk.Align.CENTER

        listbox.ListBoxRow.__init__(self, image, name, button)
